package de.fitko.fitconnect.api.services;

import com.nimbusds.jose.jwk.RSAKey;
import de.fitko.fitconnect.api.domain.auth.OAuthToken;
import de.fitko.fitconnect.api.domain.model.metadata.Metadata;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.Attachment;
import de.fitko.fitconnect.api.domain.model.metadata.data.Data;
import de.fitko.fitconnect.api.domain.model.submission.Submission;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionForPickup;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionSubmit;
import de.fitko.fitconnect.api.domain.validation.ValidationResult;
import de.fitko.fitconnect.api.exceptions.AuthenticationException;

import java.util.List;
import java.util.UUID;

/**
 * A technical system that accepts submissions on the administration side.
 *
 * @see <a href="https://docs.fitko.de/fit-connect/docs/receiving/overview">Receiving Submissions</a>
 */
public interface Subscriber {

    /**
     * Authenticates the subscriber against the FIT-Connect API and retrieves an authentication token
     *
     * @param clientId     a unique identifier within the FIT-Connect platform environment
     * @param clientSecret a secret that is only known by the application and the OAuth server
     * @param scope        OAuth scope(s) that determine if a submission is accepted by the client
     * @return {@link OAuthToken} that holds the access-token
     *
     * @throws {@link AuthenticationException} if an error occurred, including the original cause
     */
    OAuthToken retrieveOAuthToken(final String clientId, final String clientSecret, final String... scope);

    /**
     * Decrypts JWE-encrypted string data.
     *
     * @param privateKey       the private key to decrypt the JWE-encrypted string
     * @param encryptedContent the content that should be decrypted
     * @return the decrypted content as byte[]
     */
    byte[] decryptStringContent(final RSAKey privateKey, final String encryptedContent);

    /**
     * Polls available {@link SubmissionSubmit}s for a given destinationId.
     *
     * @param destinationId restricts the query to a specific destination
     * @param limit         number of submissions in result (max. is 500)
     * @param offset        position in the dataset
     * @return list of found {@link SubmissionSubmit}s
     */
    List<SubmissionForPickup> pollAvailableSubmissions(UUID destinationId, int limit, int offset);

    /**
     * Gets a {@link SubmissionSubmit}.
     *
     * @param submissionId the unique identifier of a {@link SubmissionSubmit}
     * @return the requested {@link Submission}
     */
    Submission getSubmission(UUID submissionId);

    /**
     * Loads a list of {@link Attachment}s for a {@link Submission}.
     *
     * @param submissionId the unique identifier of a {@link Submission}
     * @param attachmentIds a list of {@link Attachment} UUIDs
     * @return list of {@link Attachment} objects
     */
    List<Attachment> fetchAttachments(UUID submissionId, List<UUID> attachmentIds);

    /**
     * Validates the structure and signature of a Security-Event-Token (SET).
     *
     * @param caseId unique identifier of a {@link Submission}s case
     * @returna {@link ValidationResult}, contains an error if the SET is invalid
     */
    ValidationResult validateEventLog(UUID caseId);

    /**
     * Validates the {@link Metadata} structure against a given JSON-schema to ensure its correctness.
     *
     * @param metadata the {@link Metadata} object that is validated
     * @return a {@link ValidationResult}, contains an error if the {@link Metadata} is invalid or doesn't match the schema
     */
    ValidationResult validateMetadata(Metadata metadata, String schema);

    /**
     * Validates the {@link Data}.
     *
     * @param data the received data
     * @return a {@link ValidationResult}, contains an error if the {@link Data} is invalid
     */
    ValidationResult validateData(Data data);

    /**
     * Validates the {@link Attachment}s.
     *
     * @param attachments the received list of attachments
     * @return a {@link ValidationResult}, contains an error if the {@link Attachment}s are invalid
     */
    ValidationResult validateAttachments(List<Attachment> attachments);

    /**
     * Sends a confirmation event if the received submission matches all validations.
     *
     * @see <a href="https://docs.fitko.de/fit-connect/docs/receiving/process-and-acknowledge">Process And Acknowledge</a>
     */
    void confirmValidSubmission();
}
